@@ -1,13 +1,15 @@
 getKnitrFormat <- function(default = "latex") {
   options <- opts_knit$get(c("out.format", "rmarkdown.pandoc.to"))
+  result <- default
   if (identical(options$out.format, "markdown"))
-    return(options$rmarkdown.pandoc.to)
-  if (!is.null(options$out.format))
-    return(options$out.format)
-  default
+    result <- options$rmarkdown.pandoc.to
+  else if (!is.null(options$out.format))
+    result <- options$out.format
+  if (result %in% c("html4", "html5")) result <- "html"
+  result
 }
 
-toKable <- function(table, format = getKnitrFormat(), booktabs = TRUE)
+toKable <- function(table, format = getKnitrFormat(), booktabs = TRUE, ...)
 { 
   if (!inherits(table, "tabular"))
     stop("'table' must be a 'tabular' object.")
@@ -19,11 +21,11 @@ toKable <- function(table, format = getKnitrFormat(), booktabs = TRUE)
   
   if (format == "latex") {
     save <- if (booktabs) booktabs() else table_options()
-    lines <- paste0(capture.output(latex(table)), collapse = "\n")
+    lines <- paste0(capture.output(latex(table, ...)), collapse = "\n")
     table_options(save)
   } else
-    lines <- paste0(capture.output(html(table)), collapse = "\n")
+    lines <- paste0(capture.output(html(table, ...)), collapse = "\n")
   structure(lines,
 	  format = format, class = "knitr_kable",
 	  n_head = nrow(attr(table, "colLabels")))
-}
\ No newline at end of file
+}
